﻿using Syncfusion.UI.Xaml.RichTextBoxAdv;
using System;
using System.Collections.Generic;
using Syncfusion.UI.Xaml.Controls.Input;
using Syncfusion.UI.Xaml.Controls.Media;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Printing;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Shapes;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Syncfusion.UI.Xaml.Controls.SfRibbon;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Printing;
using Windows.Phone.UI.Input;
using Windows.UI.ViewManagement;
using Windows.UI.Text.Core;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace DocumentEditor_UWP
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class DocumentEditor : Page
    {
        #region Fields
        Assembly execAssm = typeof(DocumentEditor).GetTypeInfo().Assembly;
        PrintDocument printDocument = null;
        IPrintDocumentSource printDocumentSource = null;
        List<BitmapImage> pageImages = new List<BitmapImage>();
        Task<bool> loadAsync = null;
        CancellationTokenSource cancellationTokenSource = null;
        // For phone
        Windows.UI.ViewManagement.InputPane inputPane = null;
        StatusBar statusBar = null;
        bool isRibbonTabPanelOpened = false, isRibbonBackstageOpened = false;
        CoreTextServicesManager coreTextServiceManager = null;
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentEditor"/> class.
        /// </summary>
        public DocumentEditor()
        {
            this.InitializeComponent();
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.GettingStarted.docx");
            this.Loaded += DocumentEditor_Loaded;
            this.Unloaded += DocumentEditor_Unloaded;
            coreTextServiceManager = CoreTextServicesManager.GetForCurrentView();
            coreTextServiceManager.InputLanguageChanged += CoreTextServiceManager_InputLanguageChanged;
            this.richTextBox.SelectionChanged += RichTextBox_SelectionChanged;
            this.richTextBox.ContentChanged += RichTextBox_ContentChanged;
            this.richTextBox.RequestNavigate += RichTextBoxAdv_RequestNavigate;
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
            {
                statusBar = StatusBar.GetForCurrentView();
                inputPane = Windows.UI.ViewManagement.InputPane.GetForCurrentView();
                HardwareButtons.BackPressed += HardwareButtons_BackPressed;
                richTextBox.IsTextPredictionEnabled = true;
                richTextBox.GotFocus += RichTextBox_GotFocus;
                richTextBox.Tapped += RichTextBox_Tapped;
                inputPane.Showing += InputPane_Showing;
                inputPane.Hiding += InputPane_Hiding;
                inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.GettingStarted_Phone.docx");
                ribbon.RibbonTabPanelOpened += Ribbon_RibbonTabPanelOpened;
                ribbon.RibbonTabPanelClosed += Ribbon_RibbonTabPanelClosed;
                ribbon.BackStageOpened += Ribbon_BackStageOpened;
                ribbon.BackStageClosed += Ribbon_BackStageClosed;
            }
            else
            {
                this.highlightcolorpicker.HighlightColorGridView.SetBinding(GridView.SelectedIndexProperty, new Binding() { Source = richTextBox, Path = new PropertyPath("Selection.CharacterFormat.HighlightColor"), Mode = BindingMode.TwoWay, Converter = new HighlightColorConverter() });
                this.highlightcolorpicker.HighlightColorGridView.SelectionChanged += HighlightColorGridView_SelectionChanged;
            }
            this.richTextBox.Load(inputStream, FormatType.Docx);
            this.richTextBox.DocumentTitle = "GettingStarted";
        }
        #endregion

        #region Events
        /// <summary>
        /// Handles the Click event of the Button control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void BackButton_Click(object sender, RoutedEventArgs e)
        {
        }
        /// <summary>
        /// Handles the Loaded event of the DocumentEditor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        void DocumentEditor_Loaded(object sender, RoutedEventArgs e)
        {
            textBox.Visibility = Visibility.Collapsed;
            richTextBox.Focus(FocusState.Pointer);
        }
        /// <summary>
        /// Handles the Unloaded event of the DocumentEditor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        async void DocumentEditor_Unloaded(object sender, RoutedEventArgs e)
        {
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
            {
                statusBar.BackgroundOpacity = 0;
                this.statusBar = null;
                HardwareButtons.BackPressed -= HardwareButtons_BackPressed;
                this.inputPane.Showing -= InputPane_Showing;
                this.inputPane.Hiding -= InputPane_Hiding;
                this.inputPane = null;
                richTextBox.GotFocus -= RichTextBox_GotFocus;
                ribbon.RibbonTabPanelOpened -= Ribbon_RibbonTabPanelOpened;
                ribbon.RibbonTabPanelClosed -= Ribbon_RibbonTabPanelClosed;
                ribbon.BackStageOpened -= Ribbon_BackStageOpened;
                ribbon.BackStageClosed -= Ribbon_BackStageClosed;
            }
            this.Unloaded -= DocumentEditor_Unloaded;
            this.Loaded -= DocumentEditor_Loaded;
            if (highlightcolorpicker != null)
            {
                this.highlightcolorpicker.HighlightColorGridView.SelectionChanged -= HighlightColorGridView_SelectionChanged;
                this.highlightcolorpicker.HighlightColorGridView.SetBinding(GridView.SelectedIndexProperty, new Binding());
            }
            //Handled to cancel the asynchronous load operation.
            if (loadAsync != null && !loadAsync.IsCompleted && !loadAsync.IsFaulted && cancellationTokenSource != null)
            {
                cancellationTokenSource.Cancel();
                try
                {
                    await loadAsync;
                }
                catch
                { }
            }
            this.richTextBox.RequestNavigate -= RichTextBoxAdv_RequestNavigate;
            this.richTextBox.SelectionChanged -= RichTextBox_SelectionChanged;
            this.richTextBox.ContentChanged -= RichTextBox_ContentChanged;
            //Disposes the SfRichTextBoxAdv contents explicitly.
            this.richTextBox.Dispose();
            printDocumentSource = null;
            this.richTextBox = null;
        }
        /// <summary>
        /// Called when the input language chanages.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        async private void CoreTextServiceManager_InputLanguageChanged(CoreTextServicesManager sender, object args)
        {
            textBox.Visibility = Visibility.Visible;
            textBox.Focus(FocusState.Pointer);
            await Task.Delay(10);
            richTextBox.Focus(FocusState.Pointer);
            textBox.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Phone Events
        /// <summary>
        /// Handles the BackPressed event of the HardwareButtons control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="BackPressedEventArgs"/> instance containing the event data.</param>
        void HardwareButtons_BackPressed(object sender, BackPressedEventArgs e)
        {
            if (isRibbonTabPanelOpened)
                ribbon.HideRibbonTabPanel();
            else if (isRibbonBackstageOpened)
                ribbon.CloseBackStage();
            else if (!richTextBox.IsReadOnly)
            {
                richTextBox.IsReadOnly = true;
                richTextBox.Focus(FocusState.Pointer);
            }
            e.Handled = true;
        }
        /// <summary>
        /// Inputs the pane_ hiding.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="args">The <see cref="InputPaneVisibilityEventArgs"/> instance containing the event data.</param>
        void InputPane_Hiding(Windows.UI.ViewManagement.InputPane sender, InputPaneVisibilityEventArgs args)
        {
            inputPaneGrid.Height = 0;
        }
        /// <summary>
        /// Inputs the pane_ showing.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="args">The <see cref="InputPaneVisibilityEventArgs"/> instance containing the event data.</param>
        void InputPane_Showing(Windows.UI.ViewManagement.InputPane sender, InputPaneVisibilityEventArgs args)
        {
            inputPaneGrid.Height = sender.OccludedRect.Height;
        }
        #endregion

        #region Override Methods
        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (!Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
                RegisterForPrinting();
        }
        /// <summary>
        /// Invoked immediately after the Page is unloaded and is no longer the current source of a parent Frame.
        /// </summary>
        /// <param name="e">Event data that can be examined by overriding code. The event data is representative of the navigation that has unloaded the current Page.</param>
        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            if (!Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
                UnregisterPrinting();
        }
        #endregion

        #region RichTextBoxAdv Events
        /// <summary>
        /// Called when RichTextBoxAdv got focus.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RichTextBox_GotFocus(object sender, RoutedEventArgs e)
        {
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
                ribbon.HideRibbonTabPanel();
        }
        /// <summary>
        /// Called when RichtextBox tapped.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RichTextBox_Tapped(object sender, TappedRoutedEventArgs e)
        {
            ribbon.HideRibbonTabPanel();
            ribbon.CloseBackStage();
            richTextBox.IsReadOnly = false;
            richTextBox.Focus(FocusState.Pointer);
        }
        /// <summary>
        /// Handles the SelectionChanged event of the richTextBox control.
        /// </summary>
        /// <param name="obj">The source of the event.</param>
        /// <param name="args">The <see cref="Syncfusion.UI.Xaml.RichTextBoxAdv.SelectionChangedEventArgs"/> instance containing the event data.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        void RichTextBox_SelectionChanged(object obj, Syncfusion.UI.Xaml.RichTextBoxAdv.SelectionChangedEventArgs args)
        {
        }
        /// <summary>
        /// Handles the ContentChanged event of the RichTextBox control.
        /// </summary>
        /// <param name="obj">The source of the event.</param>
        /// <param name="args">The <see cref="ContentChangedEventArgs"/> instance containing the event data.</param>
        private void RichTextBox_ContentChanged(object obj, ContentChangedEventArgs args)
        {
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons") && richTextBox.IsReadOnly)
            {
                richTextBox.IsReadOnly = false;
                richTextBox.Focus(FocusState.Pointer);
            }
        }
        /// <summary>
        /// Handles the request navigate event of the richTextBoxAdv control.
        /// </summary>
        /// <param name="obj">The source of the event.</param>
        /// <param name="args">The <see cref="RequestNavigateEventArgs"/> instance containing the event data.</param>
        void RichTextBoxAdv_RequestNavigate(object obj, RequestNavigateEventArgs args)
        {
            if (args.Hyperlink.LinkType == HyperlinkType.Webpage || args.Hyperlink.LinkType == HyperlinkType.Email)
            {
                Uri uri = new Uri(args.Hyperlink.NavigationLink);
                LaunchUri(uri);
            }
        }
        /// <summary>
        /// Launches the URI.
        /// </summary>
        /// <param name="uri">The URI.</param>
        async void LaunchUri(Uri uri)
        {
            await Windows.System.Launcher.LaunchUriAsync(uri);
        }
        #endregion

        #region BackStage Events
        /// <summary>
        /// Handles the Click event of the WordImport control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private async void WordImport_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker fileOpenPicker = new FileOpenPicker();
            fileOpenPicker.FileTypeFilter.Add(".doc");
            fileOpenPicker.FileTypeFilter.Add(".docx");
            fileOpenPicker.FileTypeFilter.Add(".rtf");
            fileOpenPicker.FileTypeFilter.Add(".htm");
            fileOpenPicker.FileTypeFilter.Add(".html");
            fileOpenPicker.FileTypeFilter.Add(".txt");
            StorageFile stgFile = await fileOpenPicker.PickSingleFileAsync();
            CloseBackStage();
            busy.Visibility = Visibility.Visible;
            bool showErrorMsg = false;
            string errormessage = string.Empty;
            if (stgFile != null)
            {
                try
                {
                    cancellationTokenSource = new CancellationTokenSource();
                    loadAsync = this.richTextBox.LoadAsync(stgFile, cancellationTokenSource.Token);
                    await loadAsync;
                }
                catch(Exception ex)
                {
                    showErrorMsg = true;
                    errormessage = ex.Message;
                }
                if (showErrorMsg)
                {
                    if (!cancellationTokenSource.IsCancellationRequested)
                    {
                        MessageDialog dialog = null;
                        if (errormessage.Contains("Access is denied"))
                            dialog = new MessageDialog("This file cannot be open because it is being used by another process");
                        else
                            dialog = new MessageDialog("The input document could not be processed completely, Could you please email the document to support@syncfusion.com for troubleshooting.");
                        await dialog.ShowAsync();
                    }
                }
            }
            if (cancellationTokenSource != null)
                cancellationTokenSource.Dispose();
            cancellationTokenSource = null;
            loadAsync = null;
            busy.Visibility = Visibility.Collapsed;
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
            { 
                richTextBox.IsReadOnly = true;
                richTextBox.Focus(FocusState.Pointer);
            }
        }
        /// <summary>
        /// Handles the Click event of the WordExport control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private async void WordExport_Click(object sender, RoutedEventArgs e)
        {
            FileSavePicker fileSavePicker = new FileSavePicker();
            fileSavePicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;
            fileSavePicker.FileTypeChoices.Add("Word Document", new List<string>() { ".docx" });
            fileSavePicker.FileTypeChoices.Add("Word 97-2003 Document", new List<string>() { ".doc" });
            fileSavePicker.FileTypeChoices.Add("Rich Text Document", new List<string>() { ".rtf" });
            fileSavePicker.FileTypeChoices.Add("Web Page", new List<string>() { ".html" });
            fileSavePicker.FileTypeChoices.Add("Text Document", new List<string>() { ".txt" });
            fileSavePicker.SuggestedFileName = "New Document";
            StorageFile stgFile = await fileSavePicker.PickSaveFileAsync();
            CloseBackStage();
            bool showErrorMsg = false;
            if (stgFile != null)
            {
                try
                {
                    await this.richTextBox.SaveAsync(stgFile);
                }
                catch
                {
                    showErrorMsg = true;
                }
                if (showErrorMsg)
                {
                    MessageDialog dialog = new MessageDialog("Could you please email the error details along with the input document to support@syncfusion.com for troubleshooting.");
                    await dialog.ShowAsync();
                }
            }
        }
        /// <summary>
        /// Handles the Click event of the TakeATourButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void TakeATourButton_Click(object sender, RoutedEventArgs e)
        {
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.Templates.TakeATour.docx");
            this.richTextBox.Load(inputStream, FormatType.Docx);
            CloseBackStage();
        }
        /// <summary>
        /// Handles the Click event of the TitleTamplateButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void TitleTemplateButton_Click(object sender, RoutedEventArgs e)
        {
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.Templates.Title.docx");
            this.richTextBox.Load(inputStream, FormatType.Docx);
            CloseBackStage();
            richTextBox.DocumentTitle = "Title";
        }
        /// <summary>
        /// Handles the Click event of the LetterTamplateButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void LetterTemplateButton_Click(object sender, RoutedEventArgs e)
        {
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.Templates.Letter.docx");
            this.richTextBox.Load(inputStream, FormatType.Docx);
            CloseBackStage();
            richTextBox.DocumentTitle = "Letter";
        }
        /// <summary>
        /// Handles the Click event of the FaxTamplateButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void FaxTemplateButton_Click(object sender, RoutedEventArgs e)
        {
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.Templates.Fax.docx");
            this.richTextBox.Load(inputStream, FormatType.Docx);
            CloseBackStage();
            richTextBox.DocumentTitle = "Fax";
        }
        /// <summary>
        /// Handles the Click event of the ToDoListTamplateButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void ToDoListTemplateButton_Click(object sender, RoutedEventArgs e)
        {
            Stream inputStream = execAssm.GetManifestResourceStream("DocumentEditor_UWP.RTE.Assets.Templates.ToDoList.docx");
            this.richTextBox.Load(inputStream, FormatType.Docx);
            CloseBackStage();
            richTextBox.DocumentTitle = "To Do List";
        }
        /// <summary>
        /// Handles the Click event of the BlankDocumentButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void BlankDocumentButton_Click(object sender, RoutedEventArgs e)
        {
            richTextBox.DocumentTitle = "Blank Document";
            CloseBackStage();
        }
        /// <summary>
        /// Handles the OnClick event of the PrintDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private async void PrintDocument_OnClick(object sender, RoutedEventArgs e)
        {
            GetPagesAsImagesAsync();
            await PrintManager.ShowPrintUIAsync();
            CloseBackStage();
        }
        /// <summary>
        /// Handles the Clicked event of the HelpButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void HelpButton_Clicked(object sender, RoutedEventArgs e)
        {
            LaunchUri(new Uri("http://help.syncfusion.com/winrt/richtextboxadv/overview"));
            CloseBackStage();
        }
        /// <summary>
        /// Handles the Click event of the ExitButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void ExitButton_Click(object sender, RoutedEventArgs e)
        {
        }
        /// <summary>
        /// Closes the back stage.
        /// </summary>
        private void CloseBackStage()
        {
            ribbon.CloseBackStage();
            richTextBox.Focus(FocusState.Pointer);
        }
        #endregion

        #region Desktop Ribbon Events
        /// <summary>
        /// Handles the Click event of the NewDocumentButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void NewDocumentButton_Click(object sender, RoutedEventArgs e)
        {
            richTextBox.DocumentTitle = "New Document";
        }
        /// <summary>
        /// Handles the SelectionChanged event of the HighlightColorGridView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Windows.UI.Xaml.Controls.SelectionChangedEventArgs"/> instance containing the event data.</param>
        void HighlightColorGridView_SelectionChanged(object sender, Windows.UI.Xaml.Controls.SelectionChangedEventArgs e)
        {
            highlightColorDropDownButton.IsDropDownOpen = false;
            richTextBox.Focus(FocusState.Pointer);
        }
        /// <summary>
        /// Handles the Click event of the BulletButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void BulletButton_Click(object sender, RoutedEventArgs e)
        {
            if (richTextBox != null && richTextBox.Selection != null)
            {
                ListAdv list = richTextBox.Selection.ParagraphFormat.GetList();
                if (list == null || (list != null && list.AbstractList.Levels[richTextBox.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevelNumber].NumberFormat != "\uf0b7"))
                {
                    list = new ListAdv(null);
                    list.AbstractList = new AbstractListAdv(null);
                    ListLevelAdv listLevel = new ListLevelAdv(list.AbstractList);
                    listLevel.ListLevelPattern = ListLevelPattern.Bullet;
                    listLevel.NumberFormat = "\uf0b7";
                    listLevel.CharacterFormat.FontFamily = new FontFamily("Symbol");
                    listLevel.ParagraphFormat.LeftIndent = 48;
                    listLevel.ParagraphFormat.FirstLineIndent = -24;
                    list.AbstractList.Levels.Add(listLevel);
                    richTextBox.Selection.ParagraphFormat.SetList(list);
                    richTextBox.Selection.ParagraphFormat.ListLevelNumber = 0;
                }
                else
                    richTextBox.Selection.ParagraphFormat.ListLevelNumber = -1;
            }
        }
        /// <summary>
        /// Handles the Click event of the NumberingButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void NumberingButton_Click(object sender, RoutedEventArgs e)
        {
            if (richTextBox != null && richTextBox.Selection != null)
            {
                ListAdv list = richTextBox.Selection.ParagraphFormat.GetList();
                if (list == null || (list != null && list.AbstractList.Levels[richTextBox.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevelNumber].NumberFormat != "%1."))
                {
                    list = new ListAdv(null);
                    list.AbstractList = new AbstractListAdv(null);
                    ListLevelAdv listLevel = new ListLevelAdv(list.AbstractList);
                    listLevel.ListLevelPattern = ListLevelPattern.Number;
                    listLevel.NumberFormat = "%1.";
                    listLevel.StartAt = 1;
                    listLevel.CharacterFormat.FontFamily = new FontFamily("Verdana");
                    listLevel.ParagraphFormat.LeftIndent = 48;
                    listLevel.ParagraphFormat.FirstLineIndent = -24;
                    list.AbstractList.Levels.Add(listLevel);
                    richTextBox.Selection.ParagraphFormat.SetList(list);
                    richTextBox.Selection.ParagraphFormat.ListLevelNumber = 0;
                }
                else
                    richTextBox.Selection.ParagraphFormat.ListLevelNumber = -1;
            }
        }
        /// <summary>
        /// Handles the Click event of the TableAddButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void TableAddButton_Click(object sender, RoutedEventArgs e)
        {
            tabledropdown.IsDropDownOpen = false;
            string tableSize = (string)rowSize.Text + "," + (string)columnSize.Text;
            richTextBox.InsertTableCommand.Execute(tableSize);
        }
        /// <summary>
        /// Handles the Click event of the TableCancelButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RoutedEventArgs"/> instance containing the event data.</param>
        private void TableCancelButton_Click(object sender, RoutedEventArgs e)
        {
            tabledropdown.IsDropDownOpen = false;
        }
        #endregion

        #region Print implementation
        /// <summary>
        /// Prints the task requested.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="PrintTaskRequestedEventArgs"/> instance containing the event data.</param>
        private void PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs e)
        {
            PrintTask printTask = null;
            printTask = e.Request.CreatePrintTask("SamplePdf", sourceRequested =>
            {
                // Print Task event handler is invoked when the print job is completed.
                printTask.Completed += printTask_Completed;
                sourceRequested.SetSource(printDocumentSource);
            });
        }
        /// <summary>
        /// Prints the task completed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="args">The <see cref="PrintTaskCompletedEventArgs"/> instance containing the event data.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        private void printTask_Completed(PrintTask sender, PrintTaskCompletedEventArgs args)
        {
            pageImages.Clear();
        }
        /// <summary>
        /// This method registers the app for printing with Windows and sets up the necessary event handlers for the print process.
        /// </summary>
        private void RegisterForPrinting()
        {
            printDocument = new PrintDocument();
            printDocumentSource = printDocument.DocumentSource;
            printDocument.Paginate += printDocument_Paginate;
            printDocument.GetPreviewPage += printDocument_GetPreviewPage;
            printDocument.AddPages += printDocument_AddPages;
            // Create a PrintManager and add a handler for printing initialization.
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested += PrintTaskRequested;
        }
        /// <summary>
        /// This method unregisters the app for printing with Windows.
        /// </summary>
        private void UnregisterPrinting()
        {
            if (printDocument == null)
                return;
            printDocument.Paginate -= printDocument_Paginate;
            printDocument.GetPreviewPage -= printDocument_GetPreviewPage;
            printDocument.AddPages -= printDocument_AddPages;

            // Remove the handler for printing initialization.
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested -= PrintTaskRequested;
        }
        /// <summary>
        /// Handles the AddPages event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="AddPagesEventArgs"/> instance containing the event data.</param>
        void printDocument_AddPages(object sender, AddPagesEventArgs e)
        {
            int pageCount = richTextBox.PageCount;
            for (int i = 0; i < pageImages.Count; i++)
            {
                Image image = new Image();
                image.Source = pageImages[i];
                printDocument.AddPage(image);
            }
            printDocument.AddPagesComplete();
        }
        /// <summary>
        /// Handles the GetPreviewPage event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GetPreviewPageEventArgs"/> instance containing the event data.</param>
        void printDocument_GetPreviewPage(object sender, GetPreviewPageEventArgs e)
        {
            PrintDocument printDocument = sender as PrintDocument;
            int currentPreviewPage = 0;
            Interlocked.Exchange(ref currentPreviewPage, e.PageNumber - 1);
            if (pageImages.Count >= e.PageNumber)
            {
                BitmapImage bitmap = pageImages[e.PageNumber - 1];
                Image image = new Image();
                image.Source = bitmap;
                printDocument.SetPreviewPage(e.PageNumber, image);
            }
        }
        /// <summary>
        /// Handles the Paginate event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="PaginateEventArgs"/> instance containing the event data.</param>
        void printDocument_Paginate(object sender, PaginateEventArgs e)
        {
            int pageCount = richTextBox.PageCount;
            PrintDocument printDocument = sender as PrintDocument;
            // Report the number of preview pages created
            printDocument.SetPreviewPageCount(pageCount, PreviewPageCountType.Intermediate);
        }
        /// <summary>
        /// Gets the pages as images asynchronous.
        /// </summary>
        async void GetPagesAsImagesAsync()
        {
            pageImages.Clear();
            int pageCount = richTextBox.PageCount;
            for (int i = 0; i < pageCount; i++)
            {
                BitmapImage bitmapImage = await richTextBox.GetPageAsImageAsync(i);
                pageImages.Add(bitmapImage);
            }
        }
        #endregion

        #region Phone Ribbon Events
        /// <summary>
        /// Called when ribbon tab panel opened.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ribbon_RibbonTabPanelOpened(object sender, EventArgs e)
        {
            isRibbonTabPanelOpened = true;
            inputPane.TryHide();
        }
        /// <summary>
        /// Called when ribbon tab panel closed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ribbon_RibbonTabPanelClosed(object sender, EventArgs e)
        {
            isRibbonTabPanelOpened = false;
        }
        /// <summary>
        /// Called when backstage opened.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ribbon_BackStageOpened(object sender, EventArgs e)
        {
            isRibbonBackstageOpened = true;
            richTextBox.IsReadOnly = true;
            richTextBox.Focus(FocusState.Pointer);
        }
        /// <summary>
        /// Called when backstage closed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Ribbon_BackStageClosed(object sender, EventArgs e)
        {
            isRibbonBackstageOpened = false;
            richTextBox.IsReadOnly = false;
            richTextBox.Focus(FocusState.Pointer);
        }
        /// <summary>
        /// Called when highlight color drop down button clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void HighlightColorButton_Click(object sender, RoutedEventArgs e)
        {
            if (richTextBox.Selection.CharacterFormat.HighlightColor.HasValue && richTextBox.Selection.CharacterFormat.HighlightColor.Value == HighlightColor.Yellow)
                richTextBox.Selection.CharacterFormat.HighlightColor = HighlightColor.NoColor;
            else
                richTextBox.Selection.CharacterFormat.HighlightColor = HighlightColor.Yellow;
        }
        /// <summary>
        /// Called when font color button is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FontColorButton_Click(object sender, RoutedEventArgs e)
        {
            if (richTextBox.Selection.CharacterFormat.HighlightColor.HasValue && richTextBox.Selection.CharacterFormat.FontColor.Value == Colors.Red)
                richTextBox.Selection.CharacterFormat.FontColor = Colors.Black;
            else
                richTextBox.Selection.CharacterFormat.FontColor = Colors.Red;
        }
        /// <summary>
        /// Called when the insert table button is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InsertTableButton_Click(object sender, RoutedEventArgs e)
        {
            richTextBox.InsertTableCommand.Execute("3,3");
        }
        /// <summary>
        /// Called when insert picture button clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void InsertPictureButton_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker fileOpenPicker = new FileOpenPicker();
            fileOpenPicker.FileTypeFilter.Add(".png");
            fileOpenPicker.FileTypeFilter.Add(".jpg");
            fileOpenPicker.FileTypeFilter.Add(".bmp");
            StorageFile stgFile = await fileOpenPicker.PickSingleFileAsync();
            if (stgFile != null)
                richTextBox.InsertPictureCommand.Execute(stgFile);
            if (Windows.Foundation.Metadata.ApiInformation.IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
            {
                richTextBox.IsReadOnly = true;
                richTextBox.Focus(FocusState.Pointer);
            }
        }
        #endregion
    }
}
